<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 28/10/14
 * Time: 14:06
 */

namespace Folha\Logic\ESocial\Importacao;

class ManipulacaoExcel
{
    private $file;

    private $eventos;
    private $resumos;
    private $registros;

    private $leiaute;

    private $lista_eventos;

    public function __construct($arquivo)
    {
        $this->file = \PHPExcel_IOFactory::load($arquivo);

        $this->eventos = array();
        $this->resumos = array();
        $this->registros = array();

        $this->leiaute = array();

        $this->lista_eventos = array();
    }

    public function preparaDados()
    {
        try {
            self::carregaEventos();
            self::carregaResumos();
            self::carregaRegistros();

            $this->leiaute = array(
                'EVENTOS' => $this->eventos,
                'RESUMOS' => $this->resumos,
                'REGISTROS' => $this->registros
            );

            return $this->leiaute;
        } catch (\Exception $e) {
            throw Exception::carregaLeiautes();
        }
    }

    private function carregaEventos()
    {
        try {
            $this->file->setActiveSheetIndexByName('Eventos');

            for ($row = 2; !is_null($this->file->getActiveSheet()->getCell('A'.$row)->getValue()); $row++) {
                $nmevento = $this->file->getActiveSheet()->getCell('A'.$row)->getValue();
                $grupo = $this->file->getActiveSheet()->getCell('B'.$row)->getValue();

                $this->eventos[] = array(
                    'DESCRICAO' => $nmevento,
                    'NOME' => substr($nmevento, 0, 6),
                    'IDGRUPEVENT' => $grupo
                );

                $this->lista_eventos[] = substr($nmevento, 0, 6);
            }
        } catch (\Exception $e) {
            throw Exception::carregaEventos();
        }
    }

    private function carregaResumos()
    {
        try {
            $this->file->setActiveSheetIndexByName('Resumos');
            $activeSheet = $this->file->getActiveSheet();

            $listaEventos = $this->lista_eventos;

            for ($row = 1; !empty($listaEventos); $row++) {
                $value = $activeSheet->getCell('A'.$row)->getValue();

                /*Extrai o nome do evento do título da tabela (Ex: S-1000 - Informações do Empregador/Contribuinte)*/
                $eventoIndex = array_search(substr($value,0,6), $listaEventos);
                if ($eventoIndex !== false) {
                    $nmevento = $listaEventos[$eventoIndex];

                    $endRow = self::percorreResumoEvento($activeSheet, $nmevento, $row);
                    $row = $endRow;

                    unset($listaEventos[$eventoIndex]);
                }
            }
        } catch (\Exception $e) {
            throw Exception::carregaResumos();
        }
    }

    private function percorreResumoEvento($activeSheet, $nmevento, $eventoRow)
    {
        /*Pula as linhas com o nome do evento, o título da tabela e os nomes das colunas*/
        $startRow = $eventoRow + 3;

        for ($row=$startRow; !is_null($activeSheet->getCell('A'.$row)->getValue()); $row++) {

            $nmReg = $activeSheet->getCell('A'.$row)->getValue();
            $this->resumos[$nmevento][] = array(
                'REGISTRO' => $activeSheet->getCell('A'.$row)->getValue(),
                'PAI' => $activeSheet->getCell('B'.$row)->getValue(),
                'NIVEL' => $activeSheet->getCell('C'.$row)->getValue(),
                'DESCRICAO' => str_replace('\n',' ',$activeSheet->getCell('D'.$row)->getValue()),
                'OCORRENCIA' => $activeSheet->getCell('E'.$row)->getValue(),
                'CHAVE' => str_replace('\n',' ',$activeSheet->getCell('F'.$row)->getValue()),
                'CONDICAO' => str_replace('\n',' ',$activeSheet->getCell('G'.$row)->getValue())
            );

            $lastRow = $row;
        }

        return $lastRow;
    }

    private function carregaRegistros()
    {
        try {
            $this->file->setActiveSheetIndexByName('Registros');
            $activeSheet = $this->file->getActiveSheet();

            $listaEventos = $this->lista_eventos;

            for ($row = 1; !empty($listaEventos); $row++) {
                $value = $activeSheet->getCell('A'.$row)->getValue();

                /*Extrai o nome do evento do título da tabela (Ex: Registros do evento S-4999 - Adesão antecipada ao eSocial)*/
                $eventoIndex = array_search(substr($value,20,6), $listaEventos);
                if ($eventoIndex !== false) {
                    $nmevento = $listaEventos[$eventoIndex];

                    $endRow = self::percorreRegistroEvento($activeSheet, $nmevento, $row);
                    $row = $endRow;

                    unset($listaEventos[$eventoIndex]);
                }
            }
        } catch (\Exception $e) {
            throw Exception::carregaRegistros();
        }
    }

    private function percorreRegistroEvento($activeSheet, $nmevento, $eventoRow)
    {
        /*Pula as linhas com o nome do evento e os nomes das colunas*/
        $startRow = $eventoRow + 2;

        for ($row=$startRow; !is_null($activeSheet->getCell('A'.$row)->getValue()); $row++) {

            $nrCampo = $activeSheet->getCell('A'.$row)->getValue();
            $this->registros[$nmevento][] = array(
                'NRCAMPO' => $activeSheet->getCell('A'.$row)->getValue(),
                'CAMPO' => $activeSheet->getCell('B'.$row)->getValue(),
                'PAI' => $activeSheet->getCell('C'.$row)->getValue(),
                'ELEMENTO' => $activeSheet->getCell('D'.$row)->getValue(),
                'TIPO' => $activeSheet->getCell('E'.$row)->getValue(),
                'OCORRENCIA' => $activeSheet->getCell('F'.$row)->getValue(),
                'TAMANHO' => $activeSheet->getCell('G'.$row)->getValue(),
                'DECIMAL' => $activeSheet->getCell('H'.$row)->getValue(),
                'DESCRICAO' => $activeSheet->getCell('I'.$row)->getValue()
            );

            $lastRow = $row;
        }

        return $lastRow;
    }
}